@@ -152,7 +152,31 @@ def keep_specific_bins(df, axis, keep, expansions={}):
     return out_df
 
 
-def combine_cols(df, format_strings):
+def filter_cols(df, items=None, like=None, regex=None, drop_not_keep=False):
+    """Filter out columns you want to keep.
+
+    Parameters:
+      items (list-like): A list of column names to filter with
+      like (str, list[string]): A string or list of strings which will filter
+            columns where they are found in the column name
+      regex (str): A regular expression to match column names to
+      drop_not_keep (bool): Inverts the selection if true so that matched columns are dropped
+    """
+    if not like or not isinstance(like, (tuple, list)):
+        df_filtered = df.filter(items=items, like=like, regex=regex)
+    elif like:
+        if items and like:
+            raise RuntimeError("Can only use one of 'items', 'like', or 'regex'")
+        filtered = [set(col for col in df.columns if i in col) for i in like]
+        filtered = set.union(*filtered)
+        df_filtered = df.filter(items=filtered, regex=regex)
+
+    if drop_not_keep:
+        return df.drop(df_filtered.columns)
+    return df_filtered
+
+
+def combine_cols(df, format_strings, as_index=[]):
     """Combine columns together using format strings"""
     logger.info("Combining columns based on: %s", str(format_strings))
     result_names = list(format_strings.keys())
@@ -166,6 +190,8 @@ def combine_cols(df, format_strings):
     results.columns = result_names
     new_df = new_df.assign(**results)
     new_df.set_index(index, inplace=True, drop=True)
+    if as_index:
+        new_df.set_index(as_index, inplace=True, append=True)
     return new_df
 
 
@@ -267,8 +293,12 @@ def stack_weights(df, drop_n_col=False):
     logger.info("Stacking weights")
     if drop_n_col:
         df.drop("n", inplace=True, axis="columns")
+    else:
+        df.set_index("n", append=True, inplace=True)
     df.columns = pd.MultiIndex.from_tuples([c.split(":") for c in df.columns], names=["systematic", ""])
     out = df.stack(0, dropna=False)
+    if not drop_n_col:
+        out.reset_index(level="n", inplace=True)
     return out
 
 
@@ -279,18 +309,37 @@ def to_datacard_inputs(df, select_data, rename_syst_vars=False):
     logger.info("Converting to datacard inputs")
     if rename_syst_vars:
         df.columns = [n.replace("_up:", "_Up:").replace("_down:", "_Down:") for n in df.columns]
-    df.set_index("n", append=True, inplace=True)
     df = stack_weights(df)
-    df.reset_index(level="n", inplace=True)
     data_mask = df.eval(select_data)
     df["content"] = df.n
     df["content"][~data_mask] = df.sumw
     df["error"] = df.content / np.sqrt(df.n)
-
     df.drop(["n", "sumw", "sumw2"], inplace=True, axis="columns")
     return df
 
 
+def generic_pandas(df, func, *args, **kwargs):
+    """
+    Apply generic pandas function to each input
+    """
+    logger.info("Apply generic pandas function")
+    return getattr(df, func)(*args, **kwargs)
+
+
+def unstack_weights(df, weight_name="systematic", includes_counts=True):
+    """
+    The inverse to stack_weights
+    """
+    logger.info("Unstacking systematics")
+    if includes_counts:
+        df = df.set_index("n", append=True)
+    df = df.unstack(weight_name)
+    df.columns = ["{1}:{0}".format(*c) if c[1] else c[0] for c in df.columns]
+    if includes_counts:
+        df = df.reset_index(level="n")
+    return df
+
+
 def assign_col(df, assignments={}, evals={}, drop_cols=[]):
     """
     Add or change columns by assigning or evaluating new ones
@@ -333,17 +382,21 @@ def multiply_values(df, constant=0, mapping={}, weight_by_dataframes=[], apply_i
         if mask.dtype.kind != "b":
             msg = "'apply_if' statement doesn't return a boolean: %s"
             raise ValueError(msg % apply_if)
+        index = df.index
         ignored = df.loc[~mask]
         df = df.loc[mask]
     if mapping:
-        raise NotImplementedError("'mapping' option not yet implemented")
+        for select, value in mapping.items():
+            df = multiply_values(df, constant=value, apply_if=select)
     if weight_by_dataframes:
         for mul_df in weight_by_dataframes:
             df = multiply_dataframe(df, mul_df)
     if constant:
-        df = df * constant
+        numeric_cols = df.select_dtypes('number')
+        df = df.assign(**numeric_cols.multiply(constant))
     if apply_if:
-        df = pd.concat([df, ignored])
+        df = pd.concat([df, ignored]).reindex(index=index)
+
     return df
 
 
